@@ -121,7 +121,7 @@ dzl_directory_reaper_add_glob (DzlDirectoryReaper *self,
     glob = "*";
 
   p.type = PATTERN_GLOB;
-  p.min_age = min_age;
+  p.min_age = ABS (min_age);
   p.glob.directory = g_object_ref (directory);
   p.glob.glob = g_strdup (glob);
 
@@ -139,7 +139,7 @@ dzl_directory_reaper_add_file (DzlDirectoryReaper *self,
   g_return_if_fail (G_IS_FILE (file));
 
   p.type = PATTERN_FILE;
-  p.min_age = min_age;
+  p.min_age = ABS (min_age);
   p.file.file = g_object_ref (file);
 
   g_array_append_val (self->patterns, p);
@@ -168,6 +168,7 @@ remove_directory_with_children (GFile         *file,
   g_debug ("Removing uri recursively \"%s\"", uri);
 
   enumerator = g_file_enumerate_children (file,
+                                          G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK","
                                           G_FILE_ATTRIBUTE_STANDARD_NAME","
                                           G_FILE_ATTRIBUTE_STANDARD_TYPE","
                                           G_FILE_ATTRIBUTE_TIME_MODIFIED,
@@ -189,10 +190,10 @@ remove_directory_with_children (GFile         *file,
   while (NULL != (infoptr = g_file_enumerator_next_file (enumerator, cancellable, &enum_error)))
     {
       g_autoptr(GFileInfo) info = infoptr;
-      const gchar *name = g_file_info_get_name (info);
-      GFile *child = g_file_get_child (file, name);
+      g_autoptr(GFile) child = g_file_enumerator_get_child (enumerator, info);
+      GFileType file_type = g_file_info_get_file_type (info);
 
-      if (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY)
+      if (!g_file_info_get_is_symlink (info) && file_type == G_FILE_TYPE_DIRECTORY)
         {
           if (!remove_directory_with_children (child, cancellable, error))
             return FALSE;
@@ -232,6 +233,7 @@ dzl_directory_reaper_execute_worker (GTask        *task,
     {
       const Pattern *p = &g_array_index (patterns, Pattern, i);
       g_autoptr(GFileInfo) info = NULL;
+      g_autoptr(GFileInfo) dir_info = NULL;
       g_autoptr(GPatternSpec) spec = NULL;
       g_autoptr(GFileEnumerator) enumerator = NULL;
       g_autoptr(GError) error = NULL;
@@ -277,8 +279,29 @@ dzl_directory_reaper_execute_worker (GTask        *task,
               break;
             }
 
+          dir_info = g_file_query_info (p->glob.directory,
+                                        G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK","
+                                        G_FILE_ATTRIBUTE_STANDARD_TYPE",",
+                                        G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                        cancellable,
+                                        &error);
+
+          if (dir_info == NULL)
+            {
+              if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
+                g_warning ("%s", error->message);
+              break;
+            }
+
+          /* Do not follow through symlinks. */
+          if (g_file_info_get_is_symlink (dir_info) ||
+              g_file_info_get_file_type (dir_info) != G_FILE_TYPE_DIRECTORY)
+            break;
+
           enumerator = g_file_enumerate_children (p->glob.directory,
+                                                  G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK","
                                                   G_FILE_ATTRIBUTE_STANDARD_NAME","
+                                                  G_FILE_ATTRIBUTE_STANDARD_TYPE","
                                                   G_FILE_ATTRIBUTE_TIME_MODIFIED,
                                                   G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
                                                   cancellable,
@@ -293,9 +316,6 @@ dzl_directory_reaper_execute_worker (GTask        *task,
 
           while (NULL != (info = g_file_enumerator_next_file (enumerator, cancellable, NULL)))
             {
-              const gchar *name;
-
-              name = g_file_info_get_name (info);
               v64 = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
 
               /* mtime is in seconds */
@@ -303,10 +323,21 @@ dzl_directory_reaper_execute_worker (GTask        *task,
 
               if (v64 < now - p->min_age)
                 {
-                  g_autoptr(GFile) file = g_file_get_child (p->glob.directory, name);
+                  g_autoptr(GFile) file = g_file_enumerator_get_child (enumerator, info);
+                  GFileType file_type = g_file_info_get_file_type (info);
 
-                  if (g_file_query_file_type (file, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, cancellable) == G_FILE_TYPE_DIRECTORY)
+                  if (g_file_info_get_is_symlink (info) || file_type != G_FILE_TYPE_DIRECTORY)
                     {
+                      if (!g_file_delete (file, cancellable, &error))
+                        {
+                          g_warning ("%s", error->message);
+                          g_clear_error (&error);
+                        }
+                    }
+                  else
+                    {
+                      g_assert (file_type == G_FILE_TYPE_DIRECTORY);
+
                       if (!remove_directory_with_children (file, cancellable, &error) ||
                           !g_file_delete (file, cancellable, &error))
                         {
@@ -314,11 +345,6 @@ dzl_directory_reaper_execute_worker (GTask        *task,
                           g_clear_error (&error);
                         }
                     }
-                  else if (!g_file_delete (file, cancellable, &error))
-                    {
-                      g_warning ("%s", error->message);
-                      g_clear_error (&error);
-                    }
                 }
 
               g_clear_object (&info);
